// 
//Code block for DirectoryNode.cs
//
using System;
using System.IO;
using System.Data;
using System.Runtime.InteropServices;

namespace Ironring.Management.MMC 
{
	/// <summary>
	/// Any node which we create should be derived from the BaseNode.
	/// This node will get displayed in the left pane of the MMC. 
	/// 
	/// </summary>
	public class DirectoryNode : BaseNode 
	{
		protected DataTable m_table = null;

		public DirectoryNode(SnapinBase snapin) : base(snapin) 
		{
		}

		/// <summary>
		/// This is the public property to get/set the datatable value.
		/// This datatable values are the ones which gets displayed in the right pane
		/// of the MMC for the respective node item selected in the left pane.
		/// </summary>
		public DataTable Table 
		{
			get { return m_table; }
			set { m_table = value; }
		}

		/// <summary>
		/// This method is called by the MMC snap to populate the data for listview 
		/// in the right pane, for the respective node item selected in the left pane.
		/// </summary>
		public override void OnShow() 
		{
			IConsole2 console = Snapin.ResultViewConsole;

			IHeaderCtrl2 header = console as IHeaderCtrl2;
			OnShowHeader(header);

			IResultData rdata = console as IResultData;
			OnShowData(rdata);

			rdata.SetViewMode((int)ViewMode.Report);
		}

		/// <summary>
		/// This method is called by the OnShow() method to display header for the listview 
		/// in the right pane, for the respective node item selected in the left pane.
		/// </summary>
		public virtual void OnShowHeader(IHeaderCtrl2 header) 
		{
			try 
			{
				if (Table != null) 
				{

					foreach (DataColumn col in Table.Columns) 
					{
						string name = col.ColumnName;
						int ord = col.Ordinal;
						header.InsertColumn(ord, name, (int)ColumnHeaderFormat.LEFT, 140);
					}
				}
			}
			catch(COMException e) 
			{
				System.Diagnostics.Debug.WriteLine(e.Message);
				throw e;
			}
			catch(Exception e) 
			{
				System.Diagnostics.Debug.WriteLine(e.Message);
				throw e;
			}
		}

		/// <summary>
		/// This method is called by the OnShow() method to display data for the listview 
		/// in the right pane, for the respective node item selected in the left pane.
		/// </summary>
		public virtual void OnShowData(IResultData ResultData) 
		{
			if (Table != null) 
			{
				int nRow = 1;

				RESULTDATAITEM rdi = new RESULTDATAITEM();
				foreach (DataRow row in Table.Rows) 
				{
					row.ToString();
					rdi.mask=(uint)RDI.STR | (uint)RDI.IMAGE | (uint)RDI.PARAM;     
					rdi.nImage=-1; 
					rdi.str=  (IntPtr)(-1);
					rdi.nCol=0;
					rdi.lParam=m_iCookie|(nRow <<  16);
					ResultData.InsertItem(ref rdi);
					nRow++;
				}
			}
		}

		/// <summary>
		/// This method is called by the MMC snap to display each listview item's value 
		/// in the right pane.
		/// </summary>
		public override void GetDisplayInfo(ref RESULTDATAITEM ResultDataItem) 
		{
			bool bCallbase = true;

			if (Table != null) 
			{
				int nRow = (ResultDataItem.lParam >> 16) - 1;
				int nCol = ResultDataItem.nCol;

				if ((ResultDataItem.mask & (uint)RDI.STR) > 0) 
				{
					string data = DisplayName;
					if (nRow >= 0 && nRow < Table.Rows.Count && nCol >= 0 && nCol < Table.Columns.Count) 
					{
						data = Table.Rows[nRow].ItemArray[nCol].ToString();
						bCallbase = false;
					}

					ResultDataItem.str = Marshal.StringToCoTaskMemUni(data);
				}
			}

			if ((ResultDataItem.mask & (uint)RDI.IMAGE) > 0) 
			{
				int offset = 1;
				if (IsUSeSmallIcons())
					offset = 0;

				ResultDataItem.nImage = (Cookie << 16) + offset;
			}

			if (bCallbase)
				base.GetDisplayInfo(ref ResultDataItem);
		}
		
		/// <summary>
		/// This method creates the datatable with  appropriate data which servers 
		/// as the datasource for the listview displayed in the right pane, for the 
		/// respective node item selected in the left pane.
		/// </summary>
		public void CreateDataTableStructure() 
		{
			DataTable dataTable = new DataTable();
			DataColumn dataColumn = new DataColumn();
			dataColumn.DataType = System.Type.GetType("System.String");
			dataColumn.ColumnName = "Name";
			dataColumn.ReadOnly = true;
			dataColumn.Unique = true;
			// Add the Column to the DataColumnCollection.
			dataTable.Columns.Add(dataColumn);

			dataColumn = new DataColumn();
			dataColumn.DataType = System.Type.GetType("System.String");
			dataColumn.ColumnName = "Last Accessed";
			dataColumn.ReadOnly = true;
			dataColumn.Unique = false;
			// Add the Column to the DataColumnCollection.
			dataTable.Columns.Add(dataColumn);

			dataColumn = new DataColumn();
			dataColumn.DataType = System.Type.GetType("System.String");
			dataColumn.ColumnName = "Last Written";
			dataColumn.ReadOnly = true;
			dataColumn.Unique = false;
			// Add the Column to the DataColumnCollection.
			dataTable.Columns.Add(dataColumn);

			// Make the ID column the primary key column.
			DataColumn[] PrimaryKeyColumns = new DataColumn[1];
			PrimaryKeyColumns[0] = dataTable.Columns["Name"];
			dataTable.PrimaryKey = PrimaryKeyColumns;
			this.Table = dataTable;
		}

		public void FillDataTableValues() 
		{
			DataRow dataRow;
			DirectoryInfo directoryInfo = new DirectoryInfo(this.Properties["NodePath"].ToString());

			foreach(FileInfo fileInfo in directoryInfo.GetFiles()) 
			{
				if(!fileInfo.Name.ToString().EndsWith(".transfer") && !fileInfo.Name.ToString().EndsWith(".config")) 
				{
					dataRow = this.Table.NewRow();
					dataRow["Name"] = fileInfo.Name;
					dataRow["Last Accessed"] = fileInfo.LastAccessTime.ToString();
					dataRow["Last Written"] = fileInfo.LastWriteTime.ToString();
					this.Table.Rows.Add(dataRow);
				}
			}
		}
	}
}
